{-# LANGUAGE DeriveGeneric, FlexibleContexts, FlexibleInstances, NamedFieldPuns, OverloadedStrings #-}
module GEODE.Metadata.ArticleRecord
  ( Work(..)
  , ArticleRecord(..)
  , relativePath
  , uid ) where

import Data.Aeson ((.=), FromJSON(..), ToJSON(..))
import Data.Aeson.KeyMap as KeyMap (fromList)
import Data.Csv (FromField(..), FromNamedRecord(..), ToNamedRecord(..), ToField(..))
import GEODE.Metadata.Record (Record(..))
import GEODE.Metadata.Types (Has(..), ToJSONObject(..))
import GEODE.Metadata.TSV.Header (DefaultFields(..), HasDefaultHeader(..))
import GEODE.Metadata.Work (Work(..))
import GHC.Generics (Generic)
import System.FilePath ((</>), (<.>))
import Text.Printf (printf)

data ArticleRecord = ArticleRecord
  { work :: Work
  , volume :: Int
  , article :: Int } deriving (Eq, Ord, Generic, Show)

instance FromNamedRecord ArticleRecord
instance ToNamedRecord ArticleRecord

instance ToJSONObject ArticleRecord where
  toJSONObject (ArticleRecord {work, volume, article}) = KeyMap.fromList
    [ ("work", toJSON work)
    , ("volume", toJSON volume)
    , ("article", toJSON article) ]
  toJSONPairs (ArticleRecord {work, volume, article}) =
      "work" .= work
    <> "volume" .= volume
    <> "article" .= article

instance FromJSON ArticleRecord

instance Record ArticleRecord where
  uid (ArticleRecord {work, volume, article}) =
    printf "%s_%d_%d" (show $ work) volume article

  relativePath (ArticleRecord {work, volume, article}) extension =
    (show work) </> ("T" <> show volume) </> (show article) <.> extension

{-
relativePath :: Has ArticleRecord a => a -> String -> FilePath
relativePath a extension =
  (show work) </> ("T" <> show volume) </> (show article) <.> extension
  where
    ArticleRecord {work, volume, article} = get a
-}

instance HasDefaultHeader ArticleRecord where
  defaultFields = DefaultFields [ "work", "volume", "article" ]
