{-# LANGUAGE ExplicitNamespaces, FlexibleInstances, MultiParamTypeClasses, TypeOperators, UndecidableInstances #-}
module GEODE.Metadata.Types
  ( Has(..)
  , ToJSONObject(..)
  , type (@)(..) ) where

import Data.Aeson (FromJSON(..), Object, Series, ToJSON(..), Value(..), pairs)
import Data.Aeson.KeyMap as Object (union)
import Data.Csv (FromNamedRecord(..), ToNamedRecord(..))
import Data.HashMap.Strict as Hash (union)

infixl 9 @
infixl 9 :@:
data a @ b = a :@: b

class Has a b where
  get :: b -> a

instance Has a a where
  get = id

instance Has a b => Has a (b @ c) where
  get (b :@: _) = get b

instance {-# OVERLAPS #-} Has b (a @ b) where
  get (_ :@: b) = b

instance (ToNamedRecord a, ToNamedRecord b) => ToNamedRecord (a @ b) where
  toNamedRecord (a :@: b) = Hash.union (toNamedRecord a) (toNamedRecord b)

instance (FromNamedRecord a, FromNamedRecord b) => FromNamedRecord (a @ b) where
  parseNamedRecord nr = (:@:) <$> parseNamedRecord nr <*> parseNamedRecord nr

class ToJSONObject a where
  toJSONObject :: a -> Object
  toJSONPairs :: a -> Series

instance {-# OVERLAPPABLE #-} ToJSONObject a => ToJSON a where
  toJSON = Object . toJSONObject
  toEncoding = pairs . toJSONPairs

instance {-# OVERLAPPABLE #-} (FromJSON a, FromJSON b) => FromJSON (a @ b) where
  parseJSON o = (:@:) <$> parseJSON o <*> parseJSON o
